/*-
 * Copyright (c) 2014-2020 Carsten Sonne Larsen <cs@innolan.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef MEM_H_INCLUDED
#define MEM_H_INCLUDED

#include <stddef.h>
#include <stdbool.h>

void InitMemSafe(void);
void FreeAllSafe(void);
void MemUsage(long *, long *, long *);
void *AllocMemTraced(size_t, const char *, const char *, int, bool);
char *StrDupTraced(const char *, const char *, const char *, int, bool);
void *MemDupTraced(const void *s1, size_t len, const char *, const char *, int, bool);
void FreeMemTraced(void *, const char *, const char *, int);

#define AllocStringSafe(size) (char *)AllocMemTraced(size, __FILE__, __FUNCTION__, __LINE__, true)
#define AllocStructSafe(type) (type *)AllocMemTraced(sizeof(type), __FILE__, __FUNCTION__, __LINE__, true)
#define StrDupSafe(string) StrDupTraced(string, __FILE__, __FUNCTION__, __LINE__, true)
#define MemDupSafe(string, len) MemDupTraced(string, len, __FILE__, __FUNCTION__, __LINE__, true)
#define FreeMemSafe(block) FreeMemTraced(block, __FILE__, __FUNCTION__, __LINE__)
#define AllocStructNoTrace(type) (type *)AllocMemTraced(sizeof(type), __FILE__, __FUNCTION__, __LINE__, false)
#define StrDupNoTrace(string) StrDupTraced(string, __FILE__, __FUNCTION__, __LINE__, false)
#define MemDupNoTrace(string, len) MemDupTraced(string, len, __FILE__, __FUNCTION__, __LINE__, false)

#endif
